      *****************************************************************
      *                                                               *
      * MODULE NAME = THINBRDG                                        *
      *                                                               *
      * VERSION: 1.2.6                                                *
      *                                                               *
      * Author: Jim Hollingsworth, IBM Global Services                *
      * Credits: Author of the sample bridge exit (DFH0CBRE).         *
      * The validation routines are used in this program.             *
      *                                                               *
      * DESCRIPTIVE NAME = Thin Bridge exit routine                   *
      *                                                               *
      * The thin bridge serves as a thin layer between the bridge     *
      * exit interface and the CICS Web Bridge exit (DFHWBLT).        *
      * This layer is used to extend the functionality of the supplied*
      * CICS Web Bridge exit without duplicating its existing logic.  *
      *                                                               *
      * This specific implementation executes routines when the       *
      * application program issues a SEND MAP. These routines write   *
      * the BMS symbolic map,now refered to as the Application Data   *
      * Structure(ADS), and the ADS Descriptor to temporary storage.  *
      * The State Management Token is used as the queue name.         *
      * The information in the TS queue can then be read by converter *
      * encode logic to extend attribute byte processing that is      *
      * normally supplied by the CICS Web Bridge exit.                *
      *                                                               *
      * Installation Instructions:                                    *
      * 1. Examine CopyBook CSC0000 (System Constants).               *
      *    Shop standards may require changing the following:         *
      *    - CSC-ERROR-LOG (TD destination for messages),             *
      *      default is CWBO                                          *
      *    - CSC-TRACE-LEVEL the default causes informational         *
      *      messages to be written to the error log                  *
      * 2. Compile the Attribute Converter and Thin Bridge with the   *
      *    supplied copybooks:                                        *
      *    - CSC0000 (System Constants)                               *
      *    - CPEM000 (Error Messages)                                 *
      * 3. Define the Attribute Converter and Thin Bridge to CICS     *
      *    - The Attribute Converter program name=ATTRBCNV and is     *
      *      defined as a normal application program.                 *
      *    - The Thin Bridge program name=THINBRDG and is defined     *
      *      using the same parameters as the supplied Web Bridge Exit*
      *      (DFHWBLT).                                               *
      *    - The bridge transactions (CWB1 - CWB8) must be changed to *
      *      point to THINBRDG instead of DFHWBLT.                    *
      *                                                               *
      * OBJECT CHARACTERISTICS                    CICS RELEASE = 0520 *
      *  CEDA  View PROGram( ATTRBCNV )                               *
      *  PROGram        : ATTRBCNV                                    *
      *  Group          : CICSWEB                                     *
      *  DEscription    : CICS Web Interface Attribute Converter      *
      *  Language       : CObol                                       *
      *  RELoad         : No                 No | Yes                 *
      *  RESident       : No                 No | Yes                 *
      *  USAge          : Normal             Normal | Transient       *
      *  USElpacopy     : No                 No | Yes                 *
      *  Status         : Enabled            Enabled | Disabled       *
      *  RSl            : 00                 0-24 | Public            *
      *  Cedf           : No                 Yes | No                 *
      *  DAtalocation   : Any                Below | Any              *
      *  EXECKey        : User               User | Cics              *
      * REMOTE ATTRIBUTES                                             *
      *  REMOTESystem   :                                             *
      *  REMOTEName     :                                             *
      *  Transid        :                                             *
      *  Status         : Enabled            Enabled | Disabled       *
      *  RSl            : 00                 0-24 | Public            *
      *  Cedf           : No                 Yes | No                 *
      *  DAtalocation   : Any                Below | Any              *
      *  EXECKey        : User               User | Cics              *
      * REMOTE ATTRIBUTES                                             *
      *  REMOTESystem   :                                             *
      *  REMOTEName     :                                             *
      *  Transid        :                                             *
      *  EXECUtionset   : Fullapi            Fullapi | Dplsubset      *
      *                                                               *
      *                                                               *
      * OBJECT CHARACTERISTICS                                        *
      *  CEDA  View PROGram( THINBRDG )                               *
      *   PROGram        : THINBRDG                                   *
      *   Group          : CICSWEB                                    *
      *   DEscription    : CICS Web Interface Bridge Exit             *
      *   Language       : CObol              CObol | Assembler       *
      *   RELoad         : No                 No | Yes                *
      *   RESident       : No                 No | Yes                *
      *   USAge          : Normal             Normal | Transient      *
      *   USElpacopy     : No                 No | Yes                *
      *   Status         : Enabled            Enabled | Disabled      *
      *   RSl            : 00                 0-24 | Public           *
      *   Cedf           : No                 Yes | No                *
      *   DAtalocation   : Any                Below | Any             *
      *   EXECKey        : Cics               User | Cics             *
      *  REMOTE ATTRIBUTES                                            *
      *   REMOTESystem   :                                            *
      *   REMOTEName     :                                            *
      *   Transid        :                                            *
      *   Status         : Enabled                                    *
      *   RSl            : 00                                         *
      *   Cedf           : No                                         *
      *   DAtalocation   : Any                                        *
      *   EXECKey        : Cics                                       *
      *  REMOTE ATTRIBUTES                                            *
      *   REMOTESystem   :                                            *
      *   REMOTEName     :                                            *
      *   Transid        :                                            *
      *   EXECUtionset   : Fullapi                                    *
      *                                                               *
      *                                                               *
      *  OBJECT CHARACTERISTICS                                       *
      *   CEDA  View TRANSaction( CWB1 )                              *
      *    TRANSaction    : CWB1                                      *
      *    Group          : CICSWEB                                   *
      *    DEscription    : CICS Web Interface 3270 Emulation Transact*
      *    PROGram        :                                           *
      *    TWasize        : 00000              0-32767                *
      *    PROFile        : DFHCICST                                  *
      *    PArtitionset   :                                           *
      *    STAtus         : Enabled            Enabled | Disabled     *
      *    PRIMedsize     : 00000              0-65520                *
      *    TASKDATALoc    : Any                Below | Any            *
      *    TASKDATAKey    : User               User | Cics            *
      *    STOrageclear   : No                 No | Yes               *
      *    RUnaway        : System             System | 0-2700000     *
      *    SHutdown       : Enabled            Disabled | Enabled     *
      *    ISolate        : Yes                Yes | No               *
      *    Brexit         : THINBRDG                                  *
      *   REMOTE ATTRIBUTES                                           *
      *    DYnamic        : No                                        *
      *    REMOTESystem   :                                           *
      *    REMOTEName     :                                           *
      *    TRProf         :                                           *
      *    Localq         :                                           *
      *   SCHEDULING                                                  *
      *    PRIOrity       : 001                                       *
      *    TClass         : No                                        *
      *    TRANClass      : DFHTCL00                                  *
      *   ALIASES                                                     *
      *    ALias          :                                           *
      *    TASKReq        :                                           *
      *    XTRanid        :                                           *
      *    TPName         :                                           *
      *                   :                                           *
      *    XTPname        :                                           *
      *                   :                                           *
      *                   :                                           *
      *   RECOVERY                                                    *
      *    DTimout        : No                                        *
      *    RESTart        : No                                        *
      *    SPurge         : Yes                                       *
      *    TPUrge         : No                                        *
      *    DUmp           : Yes                                       *
      * +  TRACe          : Yes                                       *
      *    COnfdata       : No                                        *
      *   INDOUBT ATTRIBUTES                                          *
      *    ACtion         : Backout                                   *
      *    WAIT           : Yes                                       *
      *    WAITTime       : 00 , 00 , 00                              *
      *    INdoubt        : Backout                                   *
      *   SECURITY                                                    *
      *    RESSec         : No                                        *
      * +  CMdsec         : No                                        *
      *    Extsec         : No                                        *
      *    TRANSec        : 01                                        *
      *    RSl            : 00                                        *
      *                                                               *
      *                                                               *
      * Operation:                                                    *
      *  The URL normally used for the 3270 Bridge has the following  *
      *  format:                                                      *
      *  http://..../CICS/CWBA/DFHWBTTA/transid                       *
      *       where /CICS/ indicates that no converter is used.       *
      *  The URL used for the Attribute Converter is in the following *
      *  format:                                                      *
      *  http://..../ATTRBCNV/CWBA/DFHWBTTA/transid                   *
      *       which indicates that ATTRBCNV will be used as the       *
      *       converter program.                                      *
      *                                                               *
      *****************************************************************
      * List of Abend Codes                                           *
      * Some abends do not result in the abend code returned to the   *
      * to the user due to inconsistancy in the state data.           *
      * These are indicated by Reply No                               *
      * Some abends result in an exception trace being issued.        *
      *****************************************************************
      * Abend Description of error                        Reply Trace *
      * MBR1  Invalid BRXA-HEADER                           No   11   *
      * MBR2  Invalid BRXA-TRANSACTION-AREA                 No   12   *
      * MBR3  Invalid BRXA-COMMAND-AREA                     No   13   *
      * MBR6  Invalid User area                             No   16   *
      * MBRM  Invalid BRXA - function error                      33   *
      * MBRQ  Invalid BMS  - NO ADSD PRESENT                     37   *
      * LBLT  Error on link to DFHWBLT                           38   *
      * LATC  Error on link to ATCNWBLT                          38   *
      * mbrc  Invalid BRXA - command error                       40   *
      *****************************************************************
      * Trace point descriptions:                                     *
      *   Single digit trace points are traced if XM and BR are set   *
      *   to level 2. XM=2 is required for init, term and abend       *
      *   calls, BR is required for all API calls.                    *
      *   Double digit trace points are exception traces associated   *
      *   with abend codes.                                           *
      *   Up to 4000 bytes of data is traced.                         *
      *****************************************************************
      * No.  Data traced                                       Abend  *
      *****************************************************************
      * 11 - BRXA-HEADER                                       MBR1   *
      * 12 - BRXA-TRANSACTION-AREA                             MBR2   *
      * 13 - BRXA-COMMAND-AREA                                 MBR3   *
      * 16 - USER-AREA                                         MBR6   *
      * 33 - BRXA-COMMAND-AREA                                 MBRM   *
      * 37 - BRXA-COMMAND-AREA                                 MBRQ   *
      * 38 - BRXA-COMMAND-AREA                                 LBLT   *
      * 38 - BRXA-COMMAND-AREA                                 LATC   *
      * 39 - BRXA-COMMAND-AREA                                 INQS   *
      * 40 - BRXA-COMMAND-AREA                                 MBRC   *
      *****************************************************************
      *                                                               *
      * CHANGE ACTIVITY :                                             *
      * 1. Get Transaction Server Version sooner. It was needed in    *
      *    routine 2200-XM-COMMAND-CHECK. This fixes an MBRC abend    *
      *    when running under CICS TS 1.3 and above.      08/02/99    *
      * 2. LENGTH not specified on SEND MAP causes BRXA-FROM-LEN = 0. *
      *    We must still issue WRITEQ TS                              *
      *    and use ADSD-STRUCTURE-LENGTH                  08/02/99    *
      * 3. Check for screen replica from earlier SEND MAP 09/01/99    *
      *    Requires PQ20625 for CICS TS 1.2                           *
      * 4. Fix screen replica search                      01/09/00    *
      * 5. Improve screen replica scan                    01/22/00    *
      * 6. Check for positive move length                 02/08/00    *
      * 7. Remove replica screen search                   04/06/00    *
      * 8. Remove unused source code                      05/16/00    *
      * 9. Add NOSSRANGE compile option                   02/21/01    *
      *****************************************************************
       PROCESS APOST XOPTS(APOST SP) TRUNC(BIN) NOSSRANGE
       Identification Division.
       Program-id. THINBRDG.
       Environment Division.
       Data Division.
      *****************************************************************
       Working-storage section.
      *****************************************************************

      *****************************************************************
      * BRXA Constants                                                *
      *****************************************************************
       copy DFHBRACO.

      *****************************************************************
      * BMS Constants                                                 *
      *****************************************************************
       copy DFHBMSCA.

      *****************************************************************
      * Miscellaneous local declarations                              *
      *****************************************************************
       77 WS-BRXA-MSG                 PIC XX VALUE '01'.
       77 WS-BRXA-BIND                PIC XX VALUE '02'.

       01 program-working-storage.


        03 filler                      pic x(16)
                                         value '>WORKING STORAGE'.
        03 user-area-eye               pic x(16)
                                         value '>USER-AREA======'.
        03 response.
          05 resp                      pic s9(8) comp.
          05 resp2                     pic s9(8) comp.
        03 RC                          pic s9(8) comp.
           88 RC-OK                      value 0.
           88 RC-ERROR                   value 1.
        03 WS-ADS-QNAME                PIC X(8).
        03 WS-BRXA-FROM-LEN            PIC s9(4) comp.
        03 WS-BRXA-INTO-LEN            PIC s9(4) comp.
        03 WS-SAVE-EIBRCODE            PIC X(6).
        03 WS-SAVE-EIBRESP             PIC S9(8) COMP.
        03 WS-SAVE-EIBRESP2            PIC S9(8) COMP.
        03 WS-TS-LEVEL.
           05 WS-TS-VERSION            PIC 99.
           05 WS-TS-RELEASE            PIC 99.
           05 WS-TS-MODIFICATION       PIC 99.
        03 trace-resource              pic x(8)  value 'THINBRDG'.
        03 trace-length                pic s9(4) comp.
        03 max-trace-length            pic s9(4) comp value 4000.
        03 WS-MAPFAIL                  PIC S9(4) COMP VALUE 36.
        03 WS-STATE-DATA-IDX           PIC S9(4) COMP VALUE 1.
        03 WS-SAVE-STATE-DATA-IDX      PIC S9(4) COMP VALUE 1.
        03 WS-START-MOVE-IDX           PIC S9(4) COMP VALUE 1.
        03 WS-ADS-IDX                  PIC S9(4) COMP VALUE 1.
        03 WS-MOVE-LENGTH              PIC S9(4) COMP VALUE 1.
        03 WS-FIELD-COUNT              PIC S9(4) COMP VALUE 0.
        03 WS-SCAN-LENGTH              PIC S9(4) COMP.
        03 WS-SYMBOLIC-FIELD-NAME      PIC X(80).
        03 WS-STATE-DATA-LENGTH        PIC S9(8) COMP VALUE 32000.

      *****************************************************************
       Linkage section.
      *****************************************************************

      *****************************************************************
      * BRXA Structure (this is the BREXIT commarea)                  *
      *****************************************************************
       copy DFHBRARO.
      *****************************************************************
      * Message structures                                            *
      *****************************************************************
       01 output-message-area          pic x(32760).
       01 input-message-area           pic x(32760).

       01 from-area                    pic x(32767).
      *****************************************************************
      * The user area is created by the exit progam. It is used to    *
      * store information required by the exit program.               *
      * This information needs to be saved over multiple calls to the *
      * exit by the same task. The address is saved in the BRXA.      *
      *****************************************************************
       01 into-area                    pic x(32767).
      *****************************************************************
      * The user area is created by the exit progam. It is used to    *
      * store the symbolic map input required by the application      *
      * program that issues a RECEIVE MAP.                            *
      *****************************************************************

       01 user-area.
          03 user-area-eyecatcher      pic x(16).
      *****************************************************************
      * Input message                                                 *
      *****************************************************************
          03 input-message-area-ptr    pointer.
      *****************************************************************
      * Output message                                                *
      *****************************************************************
          03 output-message-area-ptr   pointer.
      *****************************************************************
      * End of transport mechanism specific values                    *
      *****************************************************************
          03 FILLER                    PIC X(16).
          03 WS-SYMBOL-LIST-ID.
             05 WS-SYMBOL-LIST-CHARS   PIC XX.
                88 WS-SYMBOL-LIST-START VALUE 'C0'.
             05 FILLER                 PIC X(6).
          03 WS-STATE-TOKEN-START      PIC X(16).
             88 WS-FOUND-STATE-TOKEN   VALUE 'DFH_STATE_TOKEN='.
          03 WS-STATE-DATA-AREA.
             05 WS-STATE-DATA-CHAR     PIC X
                                OCCURS 32000 TIMES
                                INDEXED BY STATE-DATA-IDX.

      *****************************************************************
       PROCEDURE DIVISION.
      *****************************************************************
      *****************************************************************
      *****************************************************************
      ** MAIN PROCESSING                                             **
      *****************************************************************
      *****************************************************************
       0100-MAINLINE.

           SET RC-OK TO TRUE.

      *****************************************************************
      * Address and validate BRXA and subareas                        *
      * Note that the brxa-ads-descriptor-ptr could be zero.          *
      *****************************************************************
           SET ADDRESS OF BRXA-HEADER TO ADDRESS OF DFHCOMMAREA.
           PERFORM 1000-VALIDATE-BRXA
           THRU    1000-VALIDATE-BRXA-EXIT.

           set address of brxa-transaction-area
                       to brxa-transaction-area-ptr
           set address of brxa-command-common
                       to brxa-command-area-ptr
           set address of brxa-bms-command
                       to brxa-command-area-ptr
           set address of user-area
                       to brxa-user-area-ptr
           set address of from-area
                       to brxa-from-ptr
           set address of into-area
                       to brxa-into-ptr
           set address of input-message-area
                       to input-message-area-ptr
           set address of output-message-area
                       to output-message-area-ptr
           set address of ads-descriptor
                       to brxa-ads-descriptor-ptr

      *****************************************************************
      * Get Transaction Server Version                                *
      * This routine moved here to get TSLEVEL sooner   08/02/99      *
      *****************************************************************
           EXEC CICS INQUIRE SYSTEM
                          CICSTSLEVEL(WS-TS-LEVEL)
                          RESP(resp)
                          RESP2(resp2)
           END-EXEC.

           IF resp NOT = DFHRESP(NORMAL)
             move 'INQS' to brxa-user-abend-code
             EXEC CICS ENTER TRACENUM(39) EXCEPTION
                       RESOURCE(trace-resource)
                       FROM(brxa-command-common)
                       FROMLENGTH(length of brxa-command-common)
                       NOHANDLE
                       END-EXEC

             perform 1300-TM-ERROR-HANDLING-ROUTINE
           END-IF.

      *****************************************************************
      * End of Get Transaction Server Version                         *
      *****************************************************************

           PERFORM 1100-VALIDATE-BRXA-SUBA
           THRU    1100-VALIDATE-BRXA-SUBA-EXIT.

           move brxa-from-len to ws-brxa-from-len.
           if brxa-function-code = brxa-bms
              if brxa-command-code = brxa-send-map and
                 brxa-from-len not = ZERO and
                 brxa-ads-descriptor-ptr = NULLS

                 move 'MBRQ' to brxa-user-abend-code
                 EXEC CICS ENTER TRACENUM(37) EXCEPTION
                           RESOURCE(trace-resource)
                           FROM(brxa-command-common)
                           FROMLENGTH(brxa-command-area-len)
                           NOHANDLE
                           END-EXEC

                 EXEC CICS RETURN END-EXEC
              end-if
           end-if

      *****************************************************************
      * Process request according to function and command codes.      *
      * Set rc-error if invalid command.                              *
      *****************************************************************
           evaluate brxa-function-code
             when brxa-xm     PERFORM 2200-XM-COMMAND-CHECK
                              THRU    2200-XM-COMMAND-CHECK-EXIT
             when brxa-sync   CONTINUE
             when brxa-tc     CONTINUE
             when brxa-bms    PERFORM 2300-BMS-SEND-CHECK
                              THRU    2300-BMS-SEND-CHECK-EXIT
             when brxa-ic     CONTINUE
             WHEN WS-BRXA-MSG CONTINUE
             when other       set rc-error to TRUE
           end-evaluate.

           if rc-error
             move 'MBRM' to brxa-user-abend-code
             EXEC CICS ENTER TRACENUM(33) EXCEPTION
                       RESOURCE(trace-resource)
                       FROM(brxa-command-common)
                       FROMLENGTH(length of brxa-command-common)
                       NOHANDLE
                       END-EXEC

             perform 1300-TM-ERROR-HANDLING-ROUTINE
           end-if

      *****************************************************************
      * Get Transaction Server Version                                *
      * This code moved to get TSLEVEL sooner             08/02/99    *
      *****************************************************************
      *    EXEC CICS INQUIRE SYSTEM
      *                   CICSTSLEVEL(WS-TS-LEVEL)
      *                   RESP(resp)
      *                   RESP2(resp2)
      *    END-EXEC.

      *    IF resp NOT = DFHRESP(NORMAL)
      *      move 'INQS' to brxa-user-abend-code
      *      EXEC CICS ENTER TRACENUM(39) EXCEPTION
      *                RESOURCE(trace-resource)
      *                FROM(brxa-command-common)
      *                FROMLENGTH(length of brxa-command-common)
      *                NOHANDLE
      *                END-EXEC

      *      perform 1300-TM-ERROR-HANDLING-ROUTINE
      *    END-IF.
      *****************************************************************
      * Link to Web Bridge exit                                       *
      *****************************************************************
           IF WS-TS-VERSION < 2 AND
              WS-TS-RELEASE < 3
              EXEC CICS LINK PROGRAM('DFHWBLT')
                             COMMAREA(BRXA-HEADER)
                             LENGTH(EIBCALEN)
                             RESP(resp)
                             RESP2(resp2)
              END-EXEC
           ELSE
              EXEC CICS LINK PROGRAM('ATCNWBLT')
                             COMMAREA(BRXA-HEADER)
                             LENGTH(EIBCALEN)
                             RESP(resp)
                             RESP2(resp2)
              END-EXEC
           END-IF.
      *    move resp to brxa-resp.
      *    move resp2 to brxa-resp2.


           IF resp NOT = DFHRESP(NORMAL)
             IF WS-TS-VERSION < 2 AND
                WS-TS-RELEASE < 3
                move 'LBLT' to brxa-user-abend-code
             ELSE
                MOVE 'LATC' TO brxa-user-abend-code
             END-IF
             EXEC CICS ENTER TRACENUM(38) EXCEPTION
                       RESOURCE(trace-resource)
                       FROM(brxa-command-common)
                       FROMLENGTH(length of brxa-command-common)
                       NOHANDLE
                       END-EXEC

               perform 1300-TM-ERROR-HANDLING-ROUTINE
           END-IF.

      *****************************************************************
      * Process request according to function and command codes.      *
      * Set rc-error if invalid command.                              *
      *****************************************************************
           evaluate brxa-function-code
             when brxa-xm     CONTINUE
             when brxa-sync   CONTINUE
             when brxa-tc     CONTINUE
             when brxa-bms    PERFORM 2400-BMS-RECV-CHECK
                              THRU    2400-BMS-RECV-CHECK-EXIT
             when brxa-ic     CONTINUE
             WHEN WS-BRXA-MSG CONTINUE
             when other       set rc-error to TRUE
           end-evaluate.


      *****************************************************************
      * Insure DFHWBLT did not corrupt our LOAD-ADS-DESCRIPTOR request*
      *****************************************************************
           IF brxa-function-code = brxa-xm AND
              brxa-command-code = brxa-init
              MOVE BRXA-YES TO BRXA-LOAD-ADS-DESCRIPTOR
           END-IF.



      *****************************************************************
      * Normal return                                                 *
      *****************************************************************
           EXEC CICS RETURN END-EXEC.
           goback.

       0100-MAINLINE-EXIT.
           EXIT.

      *****************************************************************
      ** DATA VALIDATION ROUTINES                                    **
      *****************************************************************
      *****************************************************************

      *****************************************************************
      * VALIDATE-BRXA                                                 *
      *   Check the BRXA format and the version.                      *
      *****************************************************************
       1000-VALIDATE-BRXA.
           IF BRXA-HEADER-EYECATCHER NOT = BRXA-HEADER-EYE OR
              BRXA-HEADER-LENGTH     NOT = LENGTH OF BRXA-HEADER
              EXEC CICS ENTER TRACENUM(11) EXCEPTION
                        RESOURCE(trace-resource)
                        FROM(brxa-header)
                        FROMLENGTH(eibcalen)
                        NOHANDLE
                        END-EXEC
              EXEC CICS ABEND ABCODE('MBR1') END-EXEC
           END-IF.
       1000-VALIDATE-BRXA-EXIT.
           EXIT.

      *****************************************************************
      * VALIDATE-BRXA-SUBAREAS                                        *
      *   Check the BRXA command and transaction areas.               *
      *   Check the user area (except for xm init)                    *
      *****************************************************************
       1100-VALIDATE-BRXA-SUBA.
           IF BRXA-TRAN-AREA-EYECATCHER NOT = BRXA-TRAN-AREA-EYE
              EXEC CICS ENTER TRACENUM(12) EXCEPTION
                        RESOURCE(trace-resource)
                        FROM(brxa-transaction-area)
                        FROMLENGTH(length of brxa-transaction-area)
                        NOHANDLE
                        END-EXEC
              EXEC CICS ABEND ABCODE('MBR2') END-EXEC
           END-IF.
           IF BRXA-COMMAND-AREA-EYECATCHER NOT = BRXA-COMMAND-AREA-EYE
              EXEC CICS ENTER TRACENUM(13) EXCEPTION
                        RESOURCE(trace-resource)
                        FROM(brxa-command-common)
                        FROMLENGTH(length of brxa-command-common)
                        NOHANDLE
                        END-EXEC
              EXEC CICS ABEND ABCODE('MBR3') END-EXEC
           END-IF.
       1100-VALIDATE-BRXA-SUBA-EXIT.
           EXIT.


      *****************************************************************
      *    The following sections use TS/TD as the transport          *
      *    mechanism and brsd as the start data.                      *
      *    If a messaging mechanism other than TS/TD is used          *
      *    the following routines need to be modified.                *
      *****************************************************************

      *****************************************************************
      * 1300-TM-ERROR-HANDLING-ROUTINE                                *
      *                                                               *
      *   An error has occured which means we must exit immediately.  *
      *   The caller must have set brxa-user-abend-code.              *
      *****************************************************************
       1300-tm-error-handling-routine.
           EXEC CICS RETURN END-EXEC.

      *****************************************************************
      * XM-COMMAND-CHECK                                              *
      *   Select type of XM Command                                   *
      *****************************************************************
       2200-XM-COMMAND-CHECK.
           IF WS-TS-VERSION < 2 AND
              WS-TS-RELEASE < 3
              evaluate brxa-command-code
                 when brxa-init  PERFORM 3000-BRXA-INIT
                                 THRU    3000-BRXA-INIT-EXIT
                 when brxa-term  CONTINUE
                 when brxa-abend CONTINUE
                 when other      set rc-error to TRUE
              end-evaluate
           ELSE
                 evaluate brxa-command-code
                 when brxa-init  PERFORM 3000-BRXA-INIT
                                 THRU    3000-BRXA-INIT-EXIT
                 when brxa-term  CONTINUE
                 when brxa-abend CONTINUE
                 WHEN WS-BRXA-BIND CONTINUE
                 when other      set rc-error to TRUE
              end-evaluate
           END-IF.
           if rc-error
             move 'MBRC' to brxa-user-abend-code
             EXEC CICS ENTER TRACENUM(40) EXCEPTION
                       RESOURCE(trace-resource)
                       FROM(brxa-command-common)
                       FROMLENGTH(length of brxa-command-common)
                       NOHANDLE
                       END-EXEC

             perform 1300-TM-ERROR-HANDLING-ROUTINE
           end-if.
       2200-XM-COMMAND-CHECK-EXIT.
           exit.



      *****************************************************************
      * BMS-SEND-CHECK                                                *
      *   Select type of BMS command                                  *
      *****************************************************************
       2300-BMS-SEND-CHECK.
           evaluate brxa-command-code
              when brxa-receive-map   CONTINUE
              when brxa-send-map      PERFORM 3100-BMS-SEND-MAP
                                      THRU    3100-BMS-SEND-MAP-EXIT
              when brxa-send-text     CONTINUE
              when brxa-send-control  CONTINUE
              when other              set rc-error to TRUE
           end-evaluate.
       2300-BMS-SEND-CHECK-EXIT.
           EXIT.




      *****************************************************************
      * BMS-RECV-CHECK                                                *
      *   Select type of BMS command                                  *
      *****************************************************************
       2400-BMS-RECV-CHECK.
           evaluate brxa-command-code
              when brxa-receive-map   PERFORM 3200-BMS-RECV-MAP
                                      THRU    3200-BMS-RECV-MAP-EXIT
              when brxa-send-map      CONTINUE
              when brxa-send-text     CONTINUE
              when brxa-send-control  CONTINUE
              when other              set rc-error to TRUE
           end-evaluate.
       2400-BMS-RECV-CHECK-EXIT.
           EXIT.


      *****************************************************************
      * INITIALISATION-ROUTINE                                        *
      *   To get the ADSD loaded later, we must explicitly request it *
      *   here and now.                                               *
      *****************************************************************
       3000-BRXA-INIT.
           MOVE BRXA-YES TO BRXA-LOAD-ADS-DESCRIPTOR.
       3000-BRXA-INIT-EXIT.
           EXIT.

       3100-BMS-SEND-MAP.
      *****************************************************************
      * Write the symbolic map(ADS), ADS descriptor, and              *
      * BRXA-BMS-COMMAND area to temp storage                         *
      * for processing by converter encode.                           *
      *****************************************************************

      *****************************************************************
      * If SEND MAP with MAPONLY then let it go. DFHWBLT will get the *
      * initial values. Also delete the TS queue which contains the   *
      * record when RECEIVE MAP was processed.                        *
      *****************************************************************
           IF BRXA-DATA-INDICATOR = BRXA-MAPONLY
              MOVE BRXA-IDENTIFIER(1:8) TO WS-ADS-QNAME
              EXEC CICS DELETEQ TS QUEUE(WS-ADS-QNAME)
                                   RESP(resp)
                                   RESP2(resp2)
              END-EXEC
              GO TO 3100-BMS-SEND-MAP-EXIT
           END-IF.

      *****************************************************************
      * If no symbolic map input or ADSD then exit.                   *
      * Note: A 12 byte TIOAPFX is generated even if no symbolic map. *
      *****************************************************************
           IF BRXA-FROM-PTR = nulls or
      * LENGTH not specified on SEND MAP causes BRXA-FROM-LEN = 0
      * We must still issue WRITEQ TS              08/02/99
              (BRXA-FROM-LEN = ZERO AND
               ADSD-STRUCTURE-LENGTH = ZERO) OR
              BRXA-ADS-DESCRIPTOR-PTR = nulls or
              ADSD-LENGTH = ZERO
              GO TO 3100-BMS-SEND-MAP-EXIT
           END-IF.
           MOVE BRXA-IDENTIFIER(1:8) TO WS-ADS-QNAME.

      *****************************************************************
      * If SEND MAP ERASE then delete the TS queue to purge any       *
      * previous RECEIVE or SEND maps.                                *
      *****************************************************************
           IF BRXA-ERASE-INDICATOR = BRXA-ERASE
              EXEC CICS DELETEQ TS QUEUE(WS-ADS-QNAME)
                                   RESP(resp)
                                   RESP2(resp2)
              END-EXEC
           END-IF.

           EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                               FROM(BRXA-BMS-COMMAND)
                               LENGTH(LENGTH OF BRXA-BMS-COMMAND)
                               MAIN
                               RESP(resp)
                               RESP2(resp2)
           END-EXEC.

      * If LENGTH not specified on SEND MAP then use
      * ADSD-STRUCTURE-LENGTH for WRITEQ TS LENGTH 08/02/99
           IF BRXA-FROM-LEN = ZEROS
               EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                                   FROM(from-area)
                                   LENGTH(ADSD-STRUCTURE-LENGTH)
                                   MAIN
                                   RESP(resp)
                                   RESP2(resp2)
               END-EXEC
           ELSE
               EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                                   FROM(from-area)
                                   LENGTH(WS-BRXA-FROM-LEN)
                                   MAIN
                                   RESP(resp)
                                   RESP2(resp2)
               END-EXEC
           END-IF.

           EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                               FROM(ADS-DESCRIPTOR)
                               LENGTH(ADSD-LENGTH)
                               MAIN
                               RESP(resp)
                               RESP2(resp2)
           END-EXEC.

       3100-BMS-SEND-MAP-EXIT.
           EXIT.

       3200-BMS-RECV-MAP.
      *****************************************************************
      * Write the symbolic map(ADS) to temp storage.                  *
      * This will be used if application issues a SEND MAP DATAONLY   *
      * without ERASE to re-populate fields left on the screen.       *
      *****************************************************************
      *****************************************************************
      * If no symbolic map input then exit.                           *
      * Note: A 12 byte TIOAPFX is generated even if no symbolic map. *
      *****************************************************************
           IF BRXA-INTO-PTR = nulls
      * Indicate MAPFAIL if no into pointer 06/07/99
              MOVE WS-MAPFAIL TO BRXA-RESP
      *       BRXA-INTO-LEN < 13
              GO TO 3200-BMS-RECV-MAP-EXIT
           END-IF.
           MOVE EIBRCODE TO WS-SAVE-EIBRCODE.
           MOVE EIBRESP TO WS-SAVE-EIBRESP.
           MOVE EIBRESP2 TO WS-SAVE-EIBRESP2.
           set address of into-area
                       to brxa-into-ptr.

      * Indicate MAPFAIL if no into pointer 06/07/99
           IF INTO-AREA(1:BRXA-INTO-LEN) = LOW-VALUES
              MOVE WS-MAPFAIL TO BRXA-RESP
              GO TO 3200-BMS-RECV-MAP-EXIT
           END-IF.

      * Check for screen replica from earlier SEND MAP 09/01/99
      * Improve screen replica scan   01/22/00
      *    IF WS-SYMBOL-LIST-START AND
      *       WS-FOUND-STATE-TOKEN
      * Remove replica screen search 04/06/00
      *       PERFORM 3400-REPLICATE-SCREEN
      *       THRU    3400-REPLICATE-SCREEN-EXIT.
      *    END-IF.
           move brxa-into-len to ws-brxa-into-len.
           MOVE BRXA-IDENTIFIER(1:8) TO WS-ADS-QNAME.
           EXEC CICS DELETEQ TS QUEUE(WS-ADS-QNAME)
                                RESP(resp)
                                RESP2(resp2)
           END-EXEC.

           EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                               FROM(BRXA-BMS-COMMAND)
                               LENGTH(LENGTH OF BRXA-BMS-COMMAND)
                               MAIN
                               RESP(resp)
                               RESP2(resp2)
           END-EXEC.

           EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                               FROM(into-area)
                               LENGTH(WS-BRXA-INTO-LEN)
                               MAIN
                               RESP(resp)
                               RESP2(resp2)
           END-EXEC.

           EXEC CICS WRITEQ TS QUEUE(WS-ADS-QNAME)
                               FROM(ADS-DESCRIPTOR)
                               LENGTH(ADSD-LENGTH)
                               MAIN
                               RESP(resp)
                               RESP2(resp2)
           END-EXEC.

           MOVE WS-SAVE-EIBRCODE TO EIBRCODE.
           MOVE WS-SAVE-EIBRESP TO EIBRESP.
           MOVE WS-SAVE-EIBRESP2 TO EIBRESP2.
       3200-BMS-RECV-MAP-EXIT.
           EXIT.





